@@ -18,9 +18,12 @@
 
 
  
-/*
- * dont fragment with reset ???
- */
+#ifndef IP_OFFMASK
+#define IP_OFFMASK 0x1fff
+#endif
+#ifndef IP_MF
+#define IP_MF	   0x2000
+#endif
 
 int linksock = -1;
 int mac_learn_from_ip = 0;
@@ -306,7 +309,8 @@ static void __conn_add(struct packet *p, unsigned int key)
 	struct conn_info *ci;
 	struct host_info *h_src, *h_dst;
 	
-	assert(ci = malloc(sizeof(struct conn_info)));
+	ci = malloc(sizeof(struct conn_info));
+	assert(ci);
 	memset(ci, 0, sizeof(struct conn_info));
 	ci->use_count = 1;
 	pthread_mutex_init(&ci->mutex, NULL);
@@ -344,6 +348,8 @@ static void __conn_add(struct packet *p, unsigned int key)
 	memcpy(h_dst->src_mac, h_src->dst_mac, ETH_ALEN);
 	
 	hash_put(&conn_table, key, ci);
+	
+	print_new_conn_ind(1);
 }
 
 static void ack_storm_notify(struct conn_info *ci, struct user_conn_info *uci)
@@ -371,7 +377,7 @@ static void ack_storm_notify(struct conn_info *ci, struct user_conn_info *uci)
 static void conn_add_update(struct packet *p)
 {
 	static struct user_conn_info last_toadd = {0, 0, 0, 0};
-	static last_count = 0;
+	static int last_count = 0;
 	unsigned int key;
 	struct conn_info *ci;
 	struct user_conn_info uci;
@@ -681,17 +687,18 @@ static void mac_arp_learn(struct packet *p)
 	unsigned int ip;
 	char *mac;
 	struct arpeth_hdr *arpethh;
-	
+
 	arpethh = (struct arpeth_hdr *)(p->p_arph + 1);
 	
-	if (p->p_arph->ar_op == htons(ARPOP_REPLY)) {
+	if (p->p_arph->ar_op == htons(ARPOP_REPLY) ||
+	    p->p_arph->ar_op == htons(ARPOP_REQUEST)) {
 		ip = *(unsigned int *) arpethh->ar_sip;
 		mac = arpethh->ar_sha;
-	} else {
-		ip = *(unsigned int *) arpethh->ar_tip;
-		mac = arpethh->ar_tha;
+		if (memcmp(mac, p->p_ethh->h_source, ETH_ALEN) == 0)
+			mac_table_update(ip, mac);
+		else
+			fprintf(stderr, "ARP: MAC src != ARP src for host %s\n", host_lookup(ip, hl_mode));
 	}
-	mac_table_update(ip, mac);
 }
 
 static void mac_ip_learn(struct packet *p)
@@ -714,6 +721,8 @@ static void mac_ip_learn(struct packet *p)
  * 
  */
 unsigned int pkts_received = 0;
+unsigned int pkts_dropped = 0;
+unsigned int pkts_unhandled = 0;
 unsigned int bytes_received = 0;
 
 void *hunt(void *arg)
@@ -730,7 +739,7 @@ void *hunt(void *arg)
 	
 	if (verbose)
 		printf("hunt pid %d\n", getpid());
-	add_telnet_policy();
+	add_telnet_rlogin_policy();
 	if (hash_init(&conn_table, 100, (hash_equal_func)ht_eq)) { /* Initialize hash table of connections */
 		perror("hash_init");
 		exit(1);
@@ -775,11 +784,19 @@ void *hunt(void *arg)
 			/*
 			 * don't do continue or break without packet_free !!
 			 */
+			if (p->p_raw_len < 14) {
+				pkts_dropped++;
+				goto cont;
+			}
 			ALIGNPOINTERS_ETH(p, ethh);
 			p->p_ethh = ethh;
 			p->p_timestamp = time(NULL);
 			switch (ntohs(ethh->h_proto)) {
 			    case ETH_P_IP:
+				if (p->p_raw_len < 14 + 20) {
+					pkts_dropped++;
+					goto cont;
+				}
 				ALIGNPOINTERS_IP(ethh, iph);
 				p->p_iph = iph;
 			        if (in_cksum((unsigned short *) iph,
@@ -787,8 +804,17 @@ void *hunt(void *arg)
 				if (mac_learn_from_ip)
 					mac_ip_learn(p);
 				process_ip(p);
+				if ((ntohs(iph->frag_off) & IP_OFFMASK) != 0 ||
+				    (ntohs(iph->frag_off) & IP_MF)) {
+					pkts_dropped++;
+					goto cont;
+				}
 				switch (iph->protocol) {
 				    case IPPROTO_TCP:
+					if (p->p_raw_len < 14 + IP_HDR_LENGTH(iph) + 20) {
+						pkts_dropped++;
+						goto cont;
+					}
 					p->p_type = PACKET_TCP;
 					ALIGNPOINTERS_TCP(iph, p->p_hdr.p_tcph,
 							  p->p_data);
@@ -798,9 +824,14 @@ void *hunt(void *arg)
 						conn_update_table(p, ethh, iph);
 						fast_tcp_process(p);
 						process_tcp(p);
-					}
+					} else
+						pkts_dropped++;
 					break;
 				    case IPPROTO_UDP:
+					if (p->p_raw_len < 14 + IP_HDR_LENGTH(iph) + 8) {
+						pkts_dropped++;
+						goto cont;
+					}
 					p->p_type = PACKET_UDP;
 					ALIGNPOINTERS_UDP(iph, p->p_hdr.p_udph, 
 							  p->p_data);
@@ -808,25 +839,42 @@ void *hunt(void *arg)
 					process_udp(p);
 					break;
 				    case IPPROTO_ICMP:
+					if (p->p_raw_len < 14 + IP_HDR_LENGTH(iph) + 8) {
+						pkts_dropped++;
+						goto cont;
+					}
 					p->p_type = PACKET_ICMP;
 					ALIGNPOINTERS_ICMP(iph, p->p_hdr.p_icmph,
 							   p->p_data);
 					if (in_cksum((unsigned short *) p->p_hdr.p_icmph,
 					    IP_DATA_LENGTH(iph)) == 0) {
 						process_icmp(p);
-					}
+					} else
+						pkts_dropped++;
+					break;
+				    default:
+					pkts_unhandled++;
 					break;
 				}
-				}
+				} else
+					pkts_dropped++; /* bad IP checksum */
 				break;
 			    case ETH_P_ARP:
+				if (p->p_raw_len < 14 + 28) {
+					pkts_dropped++;
+					goto cont;
+				}
 				p->p_type = PACKET_ARP;
 				ALIGNPOINTERS_ARP(ethh, p->p_arph);
 				mac_arp_learn(p);
 				process_arp(p);
 				break;
+			    default:
+				pkts_unhandled++;
+				break;
 			}
 		}
+cont:
 		packet_free(p);
 	}
 	return NULL;
@@ -872,13 +920,15 @@ int conn_list(struct user_conn_info **ruci, char **rbuf, int with_mac, int with_
 		return 0;
 	}
 	if (rbuf) {
-		assert(buf = malloc(count * 512));
+		buf = malloc(count * 512);
+		assert(buf);
 		b = buf;
 	} else
 		b = buf = NULL;
-	if (ruci)
-		assert(uci = malloc(count * sizeof(struct user_conn_info)));
-	else
+	if (ruci) {
+		uci = malloc(count * sizeof(struct user_conn_info));
+		assert(uci);
+	} else
 		uci = NULL;
 	i = 0;
 	hash_iter_set(&iter, &conn_table);
