@@ -46,7 +46,7 @@
 #include "c/list.h"
 #include "c/hash.h"
 
-#define VERSION	"1.1"
+#define VERSION	"1.3"
 
 #define max(a, b)	((a) > (b) ? (a) : (b))
 #define min(a, b)	((a) > (b) ? (b) : (a))
@@ -185,6 +185,8 @@ struct packet_info {
 	struct packet_info *next;
 };
 
+struct timejob;
+
 struct arp_spoof_info {
 	unsigned char src_fake_mac[ETH_ALEN];
 	unsigned char src_mac[ETH_ALEN];
@@ -193,7 +195,12 @@ struct arp_spoof_info {
 	unsigned int dst_addr;
 
 	int use_count;
+	int lock_count;
+	int refresh;
+	struct timejob *tj_refresh;
+	struct timejob *tj_reply; 
 	struct arp_spoof_info *next;
+	pthread_cond_t  lock_cond;
 	pthread_mutex_t mutex;
 };
 
@@ -279,7 +286,8 @@ extern inline unsigned int uci_generate_key(struct user_conn_info *uci)
  */
 extern pthread_t th_hunt;
 
-extern unsigned int pkts_received, bytes_received;
+extern unsigned int pkts_received, pkts_dropped, pkts_unhandled;
+extern unsigned int bytes_received;
 
 extern int hunt_ready;
 extern pthread_mutex_t mutex_hunt_ready;
@@ -333,6 +341,10 @@ int menu_choose_host_mask_ports_dfl(char *label, unsigned int *ret_ip,
 		unsigned int *ret_mask, unsigned int *ret_ports,
 		unsigned int dfl_ip, unsigned int dfl_mask, int *dfl_ports);
 
+void clear_new_conn_ind(void);
+void print_new_conn_ind(int add_new);
+
+
 #define MODE_SRC	0
 #define MODE_DST	1
 #define MODE_BOTH	2
@@ -384,8 +396,10 @@ int rawsock(void);
 int port_match(int port, unsigned int *db_ports);
 void port_htons(unsigned int *db_ports);
 
+extern unsigned char __suggest_mac[ETH_ALEN];
 unsigned char *suggest_mac(void);
 void ctrl_c_prompt(void);
+void clear_scr(void);
 
 /*
  * resolv.c
@@ -450,11 +464,14 @@ void func_hijack_src(struct packet *p, struct conn_info *arg);
 /*
  * arphijack
  */
+#define INPUT_MODE_RAW		0
+#define INPUT_MODE_LINEECHOR	1
+#define INPUT_MODE_LINEECHO	2
 int user_arp_hijack(struct user_conn_info *uci, char *src_fake_mac,
-		    char *dst_fake_mac);
-void user_arp_hijack_done(void);
-int arp_hijack(struct conn_info *ci, char *src_fake_mac, char *dst_fake_mac);
-void arp_hijack_done(void);
+		    char *dst_fake_mac, int input_mode);
+void user_arp_hijack_done(char *src_fake_mac, char *dst_fake_mac);
+int arp_hijack(struct conn_info *ci, char *src_fake_mac, char *dst_fake_mac, int input_mode);
+void arp_hijack_done(char *src_fake_mac, char *dst_fake_mac);
 
 /*
  * synchijack
@@ -465,10 +482,11 @@ int hijack_sync(struct conn_info *ci);
 /*
  * arpspoof
  */
-
+extern unsigned char mac_broadcast[ETH_ALEN];
+extern unsigned char mac_zero[ETH_ALEN];
 struct arp_spoof_info *start_arp_spoof(unsigned int src_addr,
 				       unsigned int dst_addr,
-		    char *src_mac, char *dst_mac, char *src_fake_mac);
+		    char *src_mac, char *dst_mac, char *src_fake_mac, int refresh);
 struct arp_spoof_info *get_arp_spoof(unsigned int src_addr, unsigned int dst_addr);
 void stop_arp_spoof(struct arp_spoof_info *asi);
 void arpspoof_menu(void);
@@ -492,6 +510,10 @@ struct arp_dont_relay *arp_dont_relay_insert(
 			unsigned int src_port, unsigned int dst_port);
 void arp_dont_relay_remove(struct arp_dont_relay *adr);
 
+extern int arp_request_spoof_through_request;
+extern int arp_rr_count;
+extern int arp_spoof_switch;
+extern int arp_spoof_with_my_mac;
 
 /*
  * rstd
@@ -535,7 +557,7 @@ extern struct list l_add_policy;
 int conn_add_match(unsigned int src_addr, unsigned int dst_addr,
  		   unsigned short src_port, unsigned short dst_port);
 int conn_add_policy(struct iphdr *iph, struct tcphdr *tcph);
-void add_telnet_policy(void);
+void add_telnet_rlogin_policy(void);
 void addpolicy_list_items(void);
 void addpolicy_add_item(void);
 void addpolicy_mod_item(void);
@@ -554,6 +576,34 @@ void lines_o_press_key();
  */
 void host_up(void);
 
+/*
+ * timer.c
+ */
+
+/*
+ * returns number of seconds of next invocation
+ * 0 - unregister
+ */
+typedef int (*time_func)(void *arg, int arg_sec);
+
+struct timejob {
+	time_func 	j_func;
+	void      	*j_arg;
+	int       	j_arg_sec;
+	struct timespec j_ts;
+	
+	/* private members */
+	struct time_job *j_next;
+};
+
+void register_timejob(struct timejob *tj);
+void register_timejob_rel(struct timejob *tj, int relsec);
+void register_timejob_milsec_rel(struct timejob *tj, int milsec);
+
+void unregister_timejob(struct timejob *tj);
+void timer_init(void);
+void timer_done(void);
+
 /*
  * net.c
  * 
