@@ -55,7 +55,8 @@ int tap(char *device, int promisc_mode)
 			perror("(tap) SOCK_PACKET allocation problems [fatal]");
 	        exit(1);					           
 	}
-	strcpy(ifr.ifr_name, device);				
+	strncpy(ifr.ifr_name, device, IFNAMSIZ);
+	ifr.ifr_name[IFNAMSIZ - 1] = 0;
 	if ((ioctl(fd, SIOCGIFFLAGS, &ifr)) < 0) {    /* Get the device info */
 	        if (verbose)
 			perror("(tap) Can't get device flags [fatal]");
